"""
Helpers enums.
"""

from enum import Enum


class MultifacePolicy(Enum):
    """
    Multiple face detection policy enum.
    """

    notAllowed = 0  #: multiple face detection not allowed
    allowed = 1  #: multiple face detection allowed
    getBest = 2  #: get best detection from the image


class Estimators(Enum):
    """
    Available loop estimators
    """

    imageOrientation = "imageOrientation"  #: image orientation
    imageModification = "imageModification"  #: image modification
    peopleCount = "peopleCount"  #: people count
    dynamicRange = "dynamicRange"  #: dynamic range

    faceDetection = "faceDetection"  #: face detection
    faceWarp = "faceWarp"  #: face warp
    gazeDirection = "gaze"  #: gaze  direction
    headPose = "headPose"  #: head pose
    faceOcclusion = "face_occlusion"  #: face occlusion
    facialHair = "facial_hair"  #: facial hair
    eyes = "eyes"  #: eyes state, points
    mouthState = "mouthState"  #: mouth state
    quality = "warpQuality"  #: warp quality
    emotions = "emotions"  #: emotions
    faceDescriptor = "faceDescriptor"  #: face descriptor
    basicAttributes = "basicAttributes"  #: basic attributes (age, gender, ethnicity)
    mask = "mask"  #: mask
    glasses = "glasses"  #: glasses
    livenessV1 = "livenessV1"  #: livenessv1
    deepfake = "deepfake"  #: deepfake
    imageColorType = "image_color_type"  #: image color type
    headwear = "headwear"  #: headwear
    faceNaturalLight = "face_natural_light"  #: face natural light
    fisheye = "fisheye"  #: fisheye
    portraitStyle = "portraitStyle"
    eyebrowExpression = "eyebrow_expression"  #: eyebrow expression
    redEyes = "red_eyes"  #: red-eyes
    faceDetectionBackground = "face_detection_background"  #: face detection background
    faceLandmarks5 = "face_landmarks5"  #: face landmarks5
    faceLandmarks68 = "face_landmarks68"  #: face landmarks68
    ags = "ags"  #: ags

    bodyDetection = "body_detection"  #: body detection
    bodyWarp = "body_warp"  #: body warp
    bodyDescriptor = "body_descriptor"  #: body descriptors
    bodyAttributes = "body_attributes"  #: body attributes

    humanDetection = "huma_detection"  #: human detection (face and body)

    humanTracker = "human_tracker"  #: human tracker (face and body, only face, only body)


class LoopEstimations(Enum):
    """
    SDK Loop estimations enum. Name - estimation target.
    """

    exif = "exif"

    imageOrientation = "image_orientation"  #: image orientation
    imageModification = "image_modification"  #: image orientation
    peopleCount = "people_count"  #: people count
    dynamicRange = "dynamic_range"  #: dynamic range
    portraitStyle = "portrait_style"

    faceDetection = "face_detection"  #: face detection
    faceWarp = "face_warp"  #: face warp
    faceLandmarks5 = "face_landmarks5"  #: face landmarks68
    faceLandmarks68 = "face_landmarks68"  #: face landmarks68
    gaze = "gaze"  #: gaze direction
    headPose = "head_pose"  #: head pose
    faceOcclusion = "face_occlusion"  #: face occlusion
    facialHair = "facial_hair"  #: facial hair
    eyes = "eyes"  #: eyes state, points
    mouthAttributes = "mouth_state"  #: mouth state
    faceWarpQuality = "face_warp_quality"  #: face warp quality
    emotions = "emotions"  #: emotions
    faceDescriptor = "face_descriptor"  #: face descriptor
    basicAttributes = "basic_attributes"  #: basic attributes (age, gender, ethnicity)
    mask = "mask"  #: mask
    glasses = "glasses"  #: glasses
    livenessV1 = "livenessV1"  #: livenessV1
    deepfake = "deepfake"  #: deepfake
    imageColorType = "image_color_type"  #: image color type
    headwear = "headwear"  #: headwear
    faceNaturalLight = "face_natural_light"  #: face natural light
    fisheye = "fisheye"  #: fisheye
    eyebrowExpression = "eyebrow_expression"  #: eyebrow expression
    redEyes = "red_eyes"  #: red-eyes
    faceDetectionBackground = "face_detection_background"  #: face detection background
    ags = "ags"  #: ags background

    bodyWarp = "body_warp"  #: body warp
    bodyDetection = "body_detection"  #: body detection
    bodyDescriptor = "body_descriptor"  #: body descriptor
    bodyAttributes = "body_attributes"  #: body attributes

    humanTracker = "human_tracker"  #: body attributes
