"""Module contains body attributes estimator realization"""

from dataclasses import dataclass

from lunavl.sdk.estimators.body_estimators.body_attributes import BodyAttributes, BodyAttributesEstimator
from lunavl.sdk.estimators.body_estimators.bodywarper import BodyWarp, BodyWarpedImage
from lunavl.sdk.faceengine.engine import VLFaceEngine

from ..worker import EstimationBrokerBase, EstimatorSettingsBase, QueueEstimatorBase, createEstimator
from ...monitoring_utils.estimation_monitoring import EstimationPoint
from ...monitoring_utils.storages import ESTIMATION_MONITORING as EM


class BodyAttributesMonitoringPoint(EstimationPoint):
    """Body attributes estimation monitoring point"""

    name = "body_attributes"


@dataclass
class BodyAttributesSettings(EstimatorSettingsBase):
    """Body attributes settings"""


class BodyAttributesQueueEstimator(
    QueueEstimatorBase[BodyWarp, BodyAttributes, BodyAttributesEstimator, BodyAttributesSettings]
):
    """Body attributes estimator"""

    async def _estimate(self, dataBatch: list[BodyWarpedImage]) -> list[BodyAttributes]:
        """Estimate body attributes"""
        with EM.monitoring(BodyAttributesMonitoringPoint(batchSize=len(dataBatch))):
            return await self._estimator.estimateBatch(dataBatch, asyncEstimate=True)


class BodyAttributesEstimationBroker(
    EstimationBrokerBase[BodyWarpedImage, BodyAttributes, BodyAttributesEstimator, BodyAttributesSettings]
):
    """Body attributes estimator"""

    # broker name
    name = "body_attributes"

    def createEstimator(self, settings: BodyAttributesSettings) -> tuple[BodyAttributesEstimator, VLFaceEngine]:
        """Create mask estimator"""
        return createEstimator(settings, lambda fe, lo: fe.createBodyAttributesEstimator(launchOptions=lo))
