"""Module contains body warp estimator realization"""

from dataclasses import dataclass

from lunavl.sdk.detectors.bodydetector import BodyDetection
from lunavl.sdk.estimators.body_estimators.bodywarper import BodyWarp, BodyWarper
from lunavl.sdk.faceengine.engine import VLFaceEngine

from ..worker import EstimationBrokerBase, EstimatorSettingsBase, QueueEstimatorBase, createEstimator
from ...monitoring_utils.estimation_monitoring import EstimationPoint
from ...monitoring_utils.storages import ESTIMATION_MONITORING as EM


class BodyWarpMonitoringPoint(EstimationPoint):
    """Body warp estimation monitoring"""

    name = "body_warp"


@dataclass
class BodyWarperSettings(EstimatorSettingsBase):
    """Body warp estimator settings"""


class BodyWarpQueueEstimator(QueueEstimatorBase[BodyDetection, BodyWarp, BodyWarper, BodyWarperSettings]):
    """Body warp estimator"""

    async def _estimate(self, dataBatch: list[BodyDetection]) -> list[BodyWarp]:
        """Estimate body warp"""
        with EM.monitoring(BodyWarpMonitoringPoint(batchSize=len(dataBatch))):
            return [self._estimator.warp(detection) for detection in dataBatch]


class BodyWarpEstimationBroker(EstimationBrokerBase[BodyDetection, BodyWarp, BodyWarper, BodyWarperSettings]):
    """Body warp broker"""

    # broker name
    name = "body_warp"

    def createEstimator(self, settings: BodyWarperSettings) -> tuple[BodyWarper, VLFaceEngine]:
        """Create body warper"""
        return createEstimator(settings, lambda fe, lo: fe.createBodyWarper())
