"""Module contains ags estimator realization"""

from dataclasses import dataclass

from lunavl.sdk.detectors.facedetector import FaceDetection
from lunavl.sdk.estimators.face_estimators.ags import AGSEstimator
from lunavl.sdk.faceengine.engine import VLFaceEngine

from ..worker import EstimationBrokerBase, EstimatorSettingsBase, QueueEstimatorBase, createEstimator
from ...monitoring_utils.estimation_monitoring import EstimationPoint
from ...monitoring_utils.storages import ESTIMATION_MONITORING as EM


class AGSMonitoringPoint(EstimationPoint):
    """AGS estimation monitoring point"""

    name = "ags"


@dataclass
class AGSSettings(EstimatorSettingsBase):
    """AGS style settings"""


class AGSQueueEstimator(QueueEstimatorBase[FaceDetection, float, AGSEstimator, AGSSettings]):
    """AGS estimator"""

    async def _estimate(self, dataBatch: list[FaceDetection]) -> list[float]:
        """Estimate ags"""
        with EM.monitoring(AGSMonitoringPoint(batchSize=len(dataBatch))):
            return self._estimator.estimateBatch(dataBatch, asyncEstimate=False)


class AGSEstimationBroker(EstimationBrokerBase[FaceDetection, float, AGSEstimator, AGSSettings]):
    """AGS style broker"""

    # broker name
    name = "ags"

    def createEstimator(self, settings: AGSSettings) -> tuple[AGSSettings, VLFaceEngine]:
        """Create AGS estimator"""
        return createEstimator(
            settings, lambda engine, launchOptions: engine.createAGSEstimator(launchOptions=launchOptions)
        )
