"""Module contains emotions estimator realization"""

from dataclasses import dataclass

from lunavl.sdk.estimators.face_estimators.emotions import Emotions, EmotionsEstimator
from lunavl.sdk.estimators.face_estimators.facewarper import FaceWarpedImage
from lunavl.sdk.faceengine.engine import VLFaceEngine

from ..worker import EstimationBrokerBase, EstimatorSettingsBase, QueueEstimatorBase, createEstimator
from ...monitoring_utils.estimation_monitoring import EstimationPoint
from ...monitoring_utils.storages import ESTIMATION_MONITORING as EM


class EmotionsMonitoringPoint(EstimationPoint):
    """Emotions estimation monitoring"""

    name = "emotions"


@dataclass
class EmotionsSettings(EstimatorSettingsBase):
    """Emotions estimator settings"""


class EmotionsQueueEstimator(QueueEstimatorBase[FaceWarpedImage, Emotions, EmotionsEstimator, EmotionsSettings]):
    """Emotions estimator"""

    async def _estimate(self, dataBatch: list[FaceWarpedImage]) -> list[Emotions]:
        """Estimate emotions"""
        with EM.monitoring(EmotionsMonitoringPoint(batchSize=len(dataBatch))):
            return await self._estimator.estimateBatch(dataBatch, asyncEstimate=True)


class EmotionsEstimationBroker(
    EstimationBrokerBase[FaceWarpedImage, Emotions, EmotionsQueueEstimator, EmotionsSettings]
):
    """Emotions broker"""

    # broker name
    name = "emotions"

    def createEstimator(self, settings: EmotionsSettings) -> tuple[EmotionsEstimator, VLFaceEngine]:
        """Create emotions estimator"""
        return createEstimator(settings, lambda fe, lo: fe.createEmotionEstimator(launchOptions=lo))
