"""Module contains eyebrow expression estimator realization"""

from dataclasses import dataclass

from lunavl.sdk.estimators.face_estimators.eyebrow_expressions import EyebrowExpressionEstimator, EyebrowExpressions
from lunavl.sdk.estimators.face_estimators.facewarper import FaceWarpedImage
from lunavl.sdk.faceengine.engine import VLFaceEngine

from ..worker import EstimationBrokerBase, EstimatorSettingsBase, QueueEstimatorBase, createEstimator
from ...monitoring_utils.estimation_monitoring import EstimationPoint
from ...monitoring_utils.storages import ESTIMATION_MONITORING as EM


class EyebrowExpressionMonitoringPoint(EstimationPoint):
    """Eyebrow expression estimation monitoring point"""

    name = "eyebrow_expression"


@dataclass
class EyebrowExpressionSettings(EstimatorSettingsBase):
    """Eyebrow expression settings"""


class EyebrowExpressionQueueEstimator(
    QueueEstimatorBase[FaceWarpedImage, EyebrowExpressions, EyebrowExpressionEstimator, EyebrowExpressionSettings]
):
    """Eyebrow expression estimator"""

    async def _estimate(self, dataBatch: list[FaceWarpedImage]) -> list[EyebrowExpressions]:
        """Estimate eyebrow expressions"""
        with EM.monitoring(EyebrowExpressionMonitoringPoint(batchSize=len(dataBatch))):
            return self._estimator.estimateBatch(dataBatch)


class EyebrowExpressionEstimationBroker(
    EstimationBrokerBase[FaceWarpedImage, EyebrowExpressions, EyebrowExpressionEstimator, EyebrowExpressionSettings]
):
    """Eyebrow expression broker"""

    # broker name
    name = "eyebrow_expression"

    def createEstimator(self, settings: EyebrowExpressionSettings) -> tuple[EyebrowExpressionEstimator, VLFaceEngine]:
        """Create eyebrow expressions estimator"""
        return createEstimator(settings, lambda fe, lo: fe.createEyebrowExpressionEstimator(launchOptions=lo))
