"""Module contains eyes estimator realization"""

from dataclasses import dataclass

from lunavl.sdk.estimators.face_estimators.eyes import EyeEstimator, EyesEstimation, WarpWithLandmarks
from lunavl.sdk.faceengine.engine import VLFaceEngine

from ..worker import EstimationBrokerBase, EstimatorSettingsBase, QueueEstimatorBase, createEstimator
from ...monitoring_utils.estimation_monitoring import EstimationPoint
from ...monitoring_utils.storages import ESTIMATION_MONITORING as EM


class EyesMonitoringPoint(EstimationPoint):
    """Eyes estimation monitoring"""

    name = "eyes"


@dataclass
class EyesSettings(EstimatorSettingsBase):
    """Eyes estimator settings"""


class EyesQueueEstimator(QueueEstimatorBase[WarpWithLandmarks, EyesEstimation, EyeEstimator, EyesSettings]):
    """Eyes estimator"""

    async def _estimate(self, dataBatch: list[WarpWithLandmarks]) -> list[EyesEstimation]:
        """Estimate eyes"""
        with EM.monitoring(EyesMonitoringPoint(batchSize=len(dataBatch))):
            return await self._estimator.estimateBatch(dataBatch, asyncEstimate=True)


class EyesEstimationBroker(EstimationBrokerBase[WarpWithLandmarks, EyesEstimation, EyeEstimator, EyesSettings]):
    """Eyes broker"""

    # broker name
    name = "eyes"

    def createEstimator(self, settings: EyesSettings) -> tuple[EyeEstimator, VLFaceEngine]:
        """Create eyes estimator"""
        return createEstimator(settings, lambda fe, lo: fe.createEyeEstimator(launchOptions=lo))
