"""Module contains face natural light estimator realization"""

from dataclasses import dataclass

from lunavl.sdk.estimators.face_estimators.facewarper import FaceWarpedImage
from lunavl.sdk.estimators.face_estimators.natural_light import FaceNaturalLight, FaceNaturalLightEstimator
from lunavl.sdk.faceengine.engine import VLFaceEngine

from ..worker import EstimationBrokerBase, EstimatorSettingsBase, QueueEstimatorBase, createEstimator
from ...monitoring_utils.estimation_monitoring import EstimationPoint
from ...monitoring_utils.storages import ESTIMATION_MONITORING as EM


class FaceNaturalLightMonitoringPoint(EstimationPoint):
    """Face natural light estimation monitoring point"""

    name = "face_natural_light"


@dataclass
class FaceNaturalLightSettings(EstimatorSettingsBase):
    """Face natural light settings"""


class FaceNaturalLightQueueEstimator(
    QueueEstimatorBase[FaceWarpedImage, FaceNaturalLight, FaceNaturalLightEstimator, FaceNaturalLightSettings]
):
    """Face natural light estimator"""

    async def _estimate(self, dataBatch: list[FaceWarpedImage]) -> list[FaceNaturalLight]:
        """Estimate mouth state"""
        with EM.monitoring(FaceNaturalLightMonitoringPoint(batchSize=len(dataBatch))):
            return await self._estimator.estimateBatch(dataBatch, asyncEstimate=True)


class FaceNaturalLightEstimationBroker(
    EstimationBrokerBase[FaceWarpedImage, FaceNaturalLight, FaceNaturalLightEstimator, FaceNaturalLightSettings]
):
    """Face natural light broker"""

    # broker name
    name = "face_natural_light"

    def createEstimator(self, settings: FaceNaturalLightSettings) -> tuple[FaceNaturalLightEstimator, VLFaceEngine]:
        """Create face natural light estimator"""
        return createEstimator(settings, lambda fe, lo: fe.createFaceNaturalLightEstimator(launchOptions=lo))
