"""Module contains facial hair estimator realization"""

from dataclasses import dataclass

from lunavl.sdk.estimators.face_estimators.facewarper import FaceWarpedImage
from lunavl.sdk.estimators.face_estimators.facial_hair import FacialHair, FacialHairEstimator
from lunavl.sdk.faceengine.engine import VLFaceEngine

from ..worker import EstimationBrokerBase, EstimatorSettingsBase, QueueEstimatorBase, createEstimator
from ...monitoring_utils.estimation_monitoring import EstimationPoint
from ...monitoring_utils.storages import ESTIMATION_MONITORING as EM


class FacialHairMonitoringPoint(EstimationPoint):
    """Facial hair estimation monitoring"""

    name = "facial_hair"


@dataclass
class FacialHairSettings(EstimatorSettingsBase):
    """Facial hair estimator settings"""


class FacialHairQueueEstimator(
    QueueEstimatorBase[FaceWarpedImage, FacialHair, FacialHairEstimator, FacialHairSettings]
):
    """Facial hair estimator"""

    async def _estimate(self, dataBatch: list[FaceWarpedImage]) -> list[FacialHair]:
        """Estimate facial hair"""
        with EM.monitoring(FacialHairMonitoringPoint(batchSize=len(dataBatch))):
            return await self._estimator.estimateBatch(dataBatch, asyncEstimate=True)


class FacialHairEstimationBroker(
    EstimationBrokerBase[FaceWarpedImage, FacialHair, FacialHairQueueEstimator, FacialHairSettings]
):
    """Facial hair broker"""

    # broker name
    name = "facial_hair"

    def createEstimator(self, settings: FacialHairSettings) -> tuple[FacialHairEstimator, VLFaceEngine]:
        """Create facial hair estimator"""
        return createEstimator(settings, lambda fe, lo: fe.createFacialHairEstimator(launchOptions=lo))
