"""Module contains glasses estimator realization"""

from dataclasses import dataclass

from lunavl.sdk.estimators.face_estimators.facewarper import FaceWarpedImage
from lunavl.sdk.estimators.face_estimators.glasses import Glasses, GlassesEstimator
from lunavl.sdk.faceengine.engine import VLFaceEngine

from ..worker import EstimationBrokerBase, EstimatorSettingsBase, QueueEstimatorBase, createEstimator
from ...monitoring_utils.estimation_monitoring import EstimationPoint
from ...monitoring_utils.storages import ESTIMATION_MONITORING as EM


class GlassesMonitoringPoint(EstimationPoint):
    """Glasses estimation monitoring point"""

    name = "glasses"


@dataclass
class GlassesSettings(EstimatorSettingsBase):
    """Glasses settings"""


class GlassesQueueEstimator(QueueEstimatorBase[FaceWarpedImage, Glasses, GlassesEstimator, GlassesSettings]):
    """Glasses estimator"""

    async def _estimate(self, dataBatch: list[FaceWarpedImage]) -> list[Glasses]:
        """Estimate glasses direction"""
        with EM.monitoring(GlassesMonitoringPoint(batchSize=len(dataBatch))):
            return await self._estimator.estimateBatch(dataBatch, asyncEstimate=True)


class GlassesEstimationBroker(EstimationBrokerBase[FaceWarpedImage, Glasses, GlassesEstimator, GlassesSettings]):
    """Glasses broker"""

    # broker name
    name = "glasses"

    def createEstimator(self, settings: GlassesSettings) -> tuple[GlassesEstimator, VLFaceEngine]:
        """Create glasses estimator"""
        return createEstimator(settings, lambda fe, lo: fe.createGlassesEstimator(launchOptions=lo))
