"""Module contains headwear estimator realization"""

from dataclasses import dataclass

from lunavl.sdk.estimators.face_estimators.facewarper import FaceWarpedImage
from lunavl.sdk.estimators.face_estimators.headwear import Headwear, HeadwearEstimator
from lunavl.sdk.faceengine.engine import VLFaceEngine

from ..worker import EstimationBrokerBase, EstimatorSettingsBase, QueueEstimatorBase, createEstimator
from ...monitoring_utils.estimation_monitoring import EstimationPoint
from ...monitoring_utils.storages import ESTIMATION_MONITORING as EM


class HeadwearMonitoringPoint(EstimationPoint):
    """Headwear estimation monitoring point"""

    name = "headwear"


@dataclass
class HeadwearSettings(EstimatorSettingsBase):
    """Headwear settings"""


class HeadwearQueueEstimator(QueueEstimatorBase[FaceWarpedImage, Headwear, HeadwearEstimator, HeadwearSettings]):
    """Headwear estimator"""

    async def _estimate(self, dataBatch: list[FaceWarpedImage]) -> list[Headwear]:
        """Estimate headwear"""
        with EM.monitoring(HeadwearMonitoringPoint(batchSize=len(dataBatch))):
            return self._estimator.estimateBatch(dataBatch)


class HeadwearEstimationBroker(EstimationBrokerBase[FaceWarpedImage, Headwear, HeadwearEstimator, HeadwearSettings]):
    """Headwear broker"""

    # broker name
    name = "headwear"

    def createEstimator(self, settings: HeadwearSettings) -> tuple[HeadwearEstimator, VLFaceEngine]:
        """Create headwear estimator"""
        return createEstimator(settings, lambda fe, lo: fe.createHeadwearEstimator(launchOptions=lo))
