"""
Module contains a functional for liveness prediction by SDK estimation data
"""

from typing import Optional

from lunavl.sdk.estimators.face_estimators.livenessv1 import LivenessPrediction


class LivenessPredicted:
    """
    Mimic an LivenessV1.
    Attributes:
        score: estimation score
        prediction: liveness prediction
    """

    #: liveness real threshold from detector settings
    defaultScoreThreshold: float

    @classmethod
    def initialize(cls, realThreshold: float):
        """
        Initialize by detector settings.
        Args:
            realThreshold: liveness real threshold from detector settings
        """
        cls.defaultScoreThreshold = realThreshold

    def __init__(self, score: float, prediction: Optional[LivenessPrediction] = None):
        self.score = score
        self.prediction = prediction

    def makePrediction(self, scoreThreshold: Optional[float]):
        """Make prediction by thresholds."""
        if scoreThreshold is None:
            scoreThreshold = self.defaultScoreThreshold
        if self.score > scoreThreshold:
            self.prediction = LivenessPrediction.Real
        else:
            self.prediction = LivenessPrediction.Spoof

    def asDict(self):
        """Return LivenessV1 representation."""
        return {"prediction": self.prediction.value, "estimations": {"score": self.score}}

    def __repr__(self) -> str:
        """
        Representation.

        Returns:
            str(self.asDict())
        """
        return str(self.asDict())
