"""Module contains mask estimator realization"""

from dataclasses import dataclass

from lunavl.sdk.estimators.face_estimators.facewarper import FaceWarpedImage
from lunavl.sdk.estimators.face_estimators.mask import Mask, MaskEstimator
from lunavl.sdk.faceengine.engine import VLFaceEngine

from ..worker import EstimationBrokerBase, EstimatorSettingsBase, QueueEstimatorBase, createEstimator
from ...monitoring_utils.estimation_monitoring import EstimationPoint
from ...monitoring_utils.storages import ESTIMATION_MONITORING as EM


class MaskMonitoringPoint(EstimationPoint):
    """Mask estimation monitoring point"""

    name = "mask"


@dataclass
class MaskSettings(EstimatorSettingsBase):
    """Mask settings"""


class MaskQueueEstimator(QueueEstimatorBase[FaceWarpedImage, Mask, MaskEstimator, MaskSettings]):
    """Mask estimator"""

    async def _estimate(self, dataBatch: list[FaceWarpedImage]) -> list[Mask]:
        """Estimate mask"""
        with EM.monitoring(MaskMonitoringPoint(batchSize=len(dataBatch))):
            return await self._estimator.estimateBatch(dataBatch, asyncEstimate=True)


class MaskEstimationBroker(EstimationBrokerBase[FaceWarpedImage, Mask, MaskEstimator, MaskSettings]):
    """Mask broker"""

    # broker name
    name = "mask"

    def createEstimator(self, settings: MaskSettings) -> tuple[MaskEstimator, VLFaceEngine]:
        """Create mask estimator"""
        return createEstimator(settings, lambda fe, lo: fe.createMaskEstimator(launchOptions=lo))
