"""Module contains mouth state estimator realization"""

from dataclasses import dataclass

from lunavl.sdk.estimators.face_estimators.facewarper import FaceWarpedImage
from lunavl.sdk.estimators.face_estimators.mouth_state import MouthStateEstimator, MouthStates
from lunavl.sdk.faceengine.engine import VLFaceEngine

from ..worker import EstimationBrokerBase, EstimatorSettingsBase, QueueEstimatorBase, createEstimator
from ...monitoring_utils.estimation_monitoring import EstimationPoint
from ...monitoring_utils.storages import ESTIMATION_MONITORING as EM


class MouthStateMonitoringPoint(EstimationPoint):
    """Mouth state estimation monitoring point"""

    name = "mouth_state"


@dataclass
class MouthStatesSettings(EstimatorSettingsBase):
    """Mouth state settings"""


class MouthStatesQueueEstimator(
    QueueEstimatorBase[FaceWarpedImage, MouthStates, MouthStateEstimator, MouthStatesSettings]
):
    """Mouth state estimator"""

    async def _estimate(self, dataBatch: list[FaceWarpedImage]) -> list[MouthStates]:
        """Estimate mouth state"""
        with EM.monitoring(MouthStateMonitoringPoint(batchSize=len(dataBatch))):
            return await self._estimator.estimateBatch(dataBatch, asyncEstimate=True)


class MouthStateEstimationBroker(
    EstimationBrokerBase[FaceWarpedImage, MouthStates, MouthStateEstimator, MouthStatesSettings]
):
    """Mouth state broker"""

    # broker name
    name = "mouth_state"

    def createEstimator(self, settings: MouthStatesSettings) -> tuple[MouthStateEstimator, VLFaceEngine]:
        """Create mouth state estimator"""
        return createEstimator(settings, lambda fe, lo: fe.createMouthEstimator(launchOptions=lo))
