"""Module contains eyes estimator realization"""

from dataclasses import dataclass

from lunavl.sdk.estimators.face_estimators.eyes import WarpWithLandmarks5
from lunavl.sdk.estimators.face_estimators.red_eye import RedEyes, RedEyesEstimator
from lunavl.sdk.faceengine.engine import VLFaceEngine

from ..worker import EstimationBrokerBase, EstimatorSettingsBase, QueueEstimatorBase, createEstimator
from ...monitoring_utils.estimation_monitoring import EstimationPoint
from ...monitoring_utils.storages import ESTIMATION_MONITORING as EM


class RedEyesMonitoringPoint(EstimationPoint):
    """Red-eyes estimation monitoring"""

    name = "red_eyes"


@dataclass
class RedEyesSettings(EstimatorSettingsBase):
    """Red-eyes estimator settings"""


class RedEyesQueueEstimator(QueueEstimatorBase[WarpWithLandmarks5, RedEyes, RedEyesEstimator, RedEyesSettings]):
    """Red-eyes estimator"""

    async def _estimate(self, dataBatch: list[WarpWithLandmarks5]) -> list[RedEyes]:
        """Estimate red-eyes"""
        with EM.monitoring(RedEyesMonitoringPoint(batchSize=len(dataBatch))):
            return self._estimator.estimateBatch(dataBatch, asyncEstimate=False)


class RedEyesEstimationBroker(EstimationBrokerBase[WarpWithLandmarks5, RedEyes, RedEyesEstimator, RedEyesSettings]):
    """Red-eyes broker"""

    # broker name
    name = "red_eyes"

    def createEstimator(self, settings: RedEyesSettings) -> tuple[RedEyesEstimator, VLFaceEngine]:
        """Create eyes estimator"""
        return createEstimator(settings, lambda fe, lo: fe.createRedEyeEstimator(launchOptions=lo))
