"""Module contains image modification estimator realization"""

from dataclasses import dataclass

from lunavl.sdk.estimators.image_estimators.image_modification import ImageModification, ImageModificationEstimator
from lunavl.sdk.faceengine.engine import VLFaceEngine
from lunavl.sdk.image_utils.image import VLImage

from ..worker import EstimationBrokerBase, EstimatorSettingsBase, QueueEstimatorBase, createEstimator
from ...monitoring_utils.estimation_monitoring import EstimationPoint
from ...monitoring_utils.storages import ESTIMATION_MONITORING as EM


class ImageModificationMonitoringPoint(EstimationPoint):
    """Image modification estimation monitoring point"""

    name = "image_modification"


@dataclass
class ImageModificationSettings(EstimatorSettingsBase):
    """Image modification settings"""


class ImageModificationQueueEstimator(
    QueueEstimatorBase[VLImage, ImageModification, ImageModificationEstimator, ImageModificationSettings]
):
    """Image modification estimator"""

    async def _estimate(self, dataBatch: list[VLImage]) -> list[ImageModification]:
        """Estimate image modification"""
        with EM.monitoring(ImageModificationMonitoringPoint(batchSize=len(dataBatch))):
            modifications: list[ImageModification] = await self._estimator.estimateBatch(dataBatch, asyncEstimate=True)
            return modifications


class ImageModificationEstimationBroker(
    EstimationBrokerBase[VLImage, ImageModification, ImageModificationEstimator, ImageModificationSettings]
):
    """Image modification broker"""

    # broker name
    name = "image_modification"

    def createEstimator(self, settings: ImageModificationSettings) -> tuple[ImageModificationEstimator, VLFaceEngine]:
        """Create image modification estimator"""
        return createEstimator(settings, lambda fe, lo: fe.createImageModificationEstimator(launchOptions=lo))
