"""Module contains image orientation estimator realization"""

from dataclasses import dataclass

from lunavl.sdk.estimators.image_estimators.orientation_mode import OrientationModeEstimator, OrientationType
from lunavl.sdk.faceengine.engine import VLFaceEngine
from lunavl.sdk.image_utils.image import VLImage

from ..worker import EstimationBrokerBase, EstimatorSettingsBase, QueueEstimatorBase, createEstimator
from ...monitoring_utils.estimation_monitoring import EstimationPoint
from ...monitoring_utils.storages import ESTIMATION_MONITORING as EM


class ImageOrientationMonitoringPoint(EstimationPoint):
    """Image orientation estimation monitoring point"""

    name = "image_orientation"


@dataclass
class ImageOrientationSettings(EstimatorSettingsBase):
    """Image orientation settings"""


class ImageOrientationQueueEstimator(
    QueueEstimatorBase[VLImage, OrientationType, OrientationModeEstimator, ImageOrientationSettings]
):
    """Image orientation estimator"""

    async def _estimate(self, dataBatch: list[VLImage]) -> list[OrientationType]:
        """Estimate image orientation"""
        with EM.monitoring(ImageOrientationMonitoringPoint(batchSize=len(dataBatch))):
            orientations: list[OrientationType] = await self._estimator.estimateBatch(dataBatch, asyncEstimate=True)
            return orientations


class ImageOrientatioEstimationBroker(
    EstimationBrokerBase[VLImage, OrientationType, OrientationModeEstimator, ImageOrientationSettings]
):
    """Image orientation broker"""

    # broker name
    name = "image_orientation"

    def createEstimator(self, settings: ImageOrientationSettings) -> tuple[OrientationModeEstimator, VLFaceEngine]:
        """Create image orientation estimator"""
        return createEstimator(settings, lambda fe, lo: fe.createOrientationModeEstimator(launchOptions=lo))
