"""Module contains runtime settings"""

from dataclasses import dataclass
from typing import Optional

from lunavl.sdk.faceengine.setting_provider import DeviceClass, RuntimeSettingsProvider, VerboseLogging


@dataclass
class RuntimeSettings:
    """
    Runtime settings
    """

    # (VerboseLogging): logging level
    logLevel: VerboseLogging = None

    # (DeviceClass): execution device type - cpu, gpu
    deviceClass: DeviceClass = DeviceClass.cpu
    # (int): number of worker threads
    numThreads: Optional[int] = None
    # (int): number of streams; Increases performance, but works only with new versions of
    #       NVIDIA driversnumber of streams; Increases performance, but works only with new
    #       versions of NVIDIA drivers
    numComputeStreams: Optional[int] = None

    def __post_init__(self):
        runtimeConf = RuntimeSettingsProvider()
        if self.logLevel is None:
            self.logLevel = runtimeConf.runtimeSettings.verboseLogging
        if self.numThreads is None:
            self.numThreads = runtimeConf.runtimeSettings.numThreads
        if self.numComputeStreams is None:
            self.numComputeStreams = runtimeConf.runtimeSettings.numComputeStreams

    def toSDKRuntimeConf(self) -> RuntimeSettingsProvider:
        """Convert runtime settings to the sdk runtime settings"""
        runtimeConf = RuntimeSettingsProvider()

        runtimeConf.runtimeSettings.verboseLogging = self.logLevel
        runtimeConf.runtimeSettings.deviceClass = self.deviceClass
        runtimeConf.runtimeSettings.numThreads = self.numThreads
        runtimeConf.runtimeSettings.numComputeStreams = self.numComputeStreams
        return runtimeConf
