"""Module contains human sample models"""

from typing import Optional

import attr

from .body_sample import AggregatedBodySample, BodySample
from .face_sample import AggregatedFaceSample, FaceSample


@attr.dataclass(slots=True)
class Sample:
    """
    A human sample - union of face sample and body samples
    """

    # face sample
    face: Optional[FaceSample] = None
    # body sample
    body: Optional[BodySample] = None


@attr.dataclass(slots=True)
class AggregatedSample:
    """
    An Aggregated human sample - union of aggregated face sample and aggregated body samples
    """

    # aggregated face sample
    face: AggregatedFaceSample = attr.field(factory=AggregatedFaceSample)
    # aggregated body sample
    body: AggregatedBodySample = attr.field(factory=AggregatedBodySample)
