"""Module contains base class for estimation monitoring points"""

from datetime import datetime
from typing import Optional


class EstimationPoint:
    """
    Estimation monitoring point, implementation Monitoring point protocol

    Attributes:
        executionTime (float): execution estimations time (batch)
        batchSize (int): batch size for estimations
        eventTime (float): event time, timestamp
    """

    __slots__ = ("executionTime", "batchSize", "eventTime")

    # associated estimation name
    name: str

    def __init__(self, eventTime: Optional[float] = None, executionTime: float = 0, batchSize: int = 0):
        self.executionTime = executionTime
        self.batchSize = batchSize
        self.eventTime = eventTime if eventTime is not None else datetime.now().timestamp()

    def getFields(self) -> dict:
        """Get monitoring fields"""
        return {"execution_time": self.executionTime}

    def getTags(self) -> dict:
        """Get monitoring tags"""
        return {"batch_size": self.batchSize}
