"""
Module contain asyncio.Queue with name
"""

from asyncio import Future, Queue
from typing import Generic, TypeVar

EstimationData = TypeVar("EstimationData")
Estimation = TypeVar("Estimation")


class QTask(Generic[EstimationData, Estimation]):
    """
    Queue task
    Attributes:
        data (EstimationData): task data
        callback (Future[Estimation]): callback for task result
    """

    __slots__ = ("data", "callback")

    def __init__(self, data: EstimationData, callback: Future[Estimation]):
        self.data = data
        self.callback = callback


class NamedQueue(Queue[QTask[EstimationData, Estimation]], Generic[EstimationData, Estimation]):
    """
    AsyncIO queue with name.

    Attributes:
        name (str): queue name
    """

    def __init__(self, *args, name: str = "", **kwargs):
        super().__init__(*args, **kwargs)
        self.name = name

    @property
    def unfinishedTasks(self) -> int:
        """Get unfinished task count"""
        return self._unfinished_tasks
